/*#############################################################
Driver name	    : TG12864H3_F4.c
Author 					: Grant Phillips
Date Modified   : 03/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)
Tested On       : STM32F4-Discovery

Description			: Provides a library to access the TG12864H3
									graphics LCD.

Requirements    : * STM32F4-Discovery Board
									* Define the SET and RESET functions for the
										output port pins.
									* Do the proper GPIO initialization for the
										output port pins in TG12864H3_Init();

Functions				: TG12864H3_Init
									TG12864H3_SetContrast
									TG12864H3_ClrScr
									TG12864H3_PowerSave
									TG12864H3_PowerActive
									TG12864H3_DrawPixel
									TG12864H3_DrawLine
									TG12864H3_DrawCircle
									TG12864H3_GotoXY
									TG12864H3_PutStr
									TG12864H3_PutChar
													  
Special Note(s) : NONE
##############################################################*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/* //////////////////////////////////////////////////////////////
Define the SET and RESET functions for each of the output port 
pins for the TG12864H3 here.  
Also do the proper GPIO initialization for these port pins in
TG12864H3_Init().
////////////////////////////////////////////////////////////// */
#define lcd_cs_1		GPIO_WriteBit(GPIOD, GPIO_Pin_7, Bit_SET)			//CS = 1
#define lcd_cs_0		GPIO_WriteBit(GPIOD, GPIO_Pin_7, Bit_RESET)		//CS = 0
#define lcd_rst_1		GPIO_WriteBit(GPIOE, GPIO_Pin_2, Bit_SET)			//RST = 1
#define lcd_rst_0		GPIO_WriteBit(GPIOE, GPIO_Pin_2, Bit_RESET)		//RST = 0
#define lcd_a0_1		GPIO_WriteBit(GPIOD, GPIO_Pin_6, Bit_SET)			//A0 = 1
#define lcd_a0_0		GPIO_WriteBit(GPIOD, GPIO_Pin_6, Bit_RESET)		//A0 = 0

#define lcd_sclk_1	GPIO_WriteBit(GPIOB, GPIO_Pin_13, Bit_SET)		//SCLK = 1
#define lcd_sclk_0	GPIO_WriteBit(GPIOB, GPIO_Pin_13, Bit_RESET)	//SCLK = 0
#define lcd_sid_1		GPIO_WriteBit(GPIOB, GPIO_Pin_15, Bit_SET)		//SID = 1
#define lcd_sid_0		GPIO_WriteBit(GPIOB, GPIO_Pin_15, Bit_RESET)	//SID = 0

#define lcd_delayx			2				//delay for writing and reading from lcd
#define lcd_init_delay	500			//delay when initializing the lcd
#define lcd_contrast		30			//contrast of the LCD (0-63)


/* TG12864H3 Command Set */
#define CMD_DISPLAY_OFF         0xAE
#define CMD_DISPLAY_ON          0xAF
#define CMD_SET_DISP_START_LINE 0x40
#define CMD_SET_PAGE            0xB0
#define CMD_SET_COLUMN_UPPER    0x10
#define CMD_SET_COLUMN_LOWER    0x00
#define CMD_SET_ADC_NORMAL      0xA0
#define CMD_SET_ADC_REVERSE     0xA1
#define CMD_SET_DISP_NORMAL     0xA6
#define CMD_SET_DISP_REVERSE    0xA7
#define CMD_SET_ALLPTS_NORMAL   0xA4
#define CMD_SET_ALLPTS_ON       0xA5
#define CMD_SET_BIAS_9          0xA2
#define CMD_SET_BIAS_7          0xA3
#define CMD_RMW                 0xE0
#define CMD_RMW_CLEAR 					0xEE
#define CMD_INTERNAL_RESET  		0xE2
#define CMD_SET_COM_NORMAL  		0xC0
#define CMD_SET_COM_REVERSE  		0xC8
#define CMD_SET_POWER_CONTROL  	0x28
#define CMD_SET_RESISTOR_RATIO  0x20
#define CMD_SET_VOLUME_FIRST  	0x81
#define CMD_SET_VOLUME_SECOND  	0
#define CMD_SET_STATIC_OFF  		0xAC
#define CMD_SET_STATIC_ON  			0xAD
#define CMD_SET_STATIC_REG  		0x0
#define CMD_SET_BOOSTER_FIRST  	0xF8
#define CMD_SET_BOOSTER_234  		0
#define CMD_SET_BOOSTER_5  			1
#define CMD_SET_BOOSTER_6  			3
#define CMD_NOP  								0xE3
#define CMD_TEST  							0xF0


void lcd_data(unsigned char dat);
void TG12864H3_Init(void);
void TG12864H3_Command(unsigned char cmd);
void TG12864H3_WriteData(unsigned char dat);
unsigned char TG12864H3_ReadData(void);
void TG12864H3_SetContrast(unsigned char value) ;
void TG12864H3_SetPage(unsigned char page);
void TG12864H3_SetColumn(unsigned char Col);
void TG12864H3_ClrScr(unsigned char color);
void TG12864H3_Delay(unsigned int del);
void TG12864H3_PowerSave(void);
void TG12864H3_PowerActive(void);
void TG12864H3_DrawPixel(unsigned char x, unsigned char y, unsigned char color);
//void TG12864H3_DrawLine(int x1, int y1, int x2, int y2, unsigned int color);
void TG12864H3_DrawLine(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, unsigned int color);
void TG12864H3_DrawCircle(unsigned char x, unsigned char y, unsigned char radius, unsigned char fill, unsigned char color);
void TG12864H3_WriteCharCol(unsigned char v, unsigned char x, unsigned char page, unsigned char color);
void TG12864H3_DrawChar(unsigned char c, unsigned char x, unsigned char page, unsigned char color);

void TG12864H3_GotoXY(unsigned char x, unsigned char y);
void TG12864H3_PutChar(unsigned char c, unsigned char color);
void TG12864H3_PutStr(char *str, unsigned char color);

unsigned char TG12864H3_DisplayRAM[128][8];
unsigned char lcd_char_x=0, lcd_char_y=0;



uint8_t TG1286H3_SPI2_send(uint8_t data) 
{   
	 SPI_I2S_SendData(SPI2, data);																	//write data to be transmitted to the SPI
	 while(!SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_TXE));				//wait until transmit complete	
	 while(!SPI_I2S_GetFlagStatus(SPI2, SPI_I2S_FLAG_RXNE));				//wait until receive complete
	 return(SPI_I2S_ReceiveData(SPI2));														//return received data from SPI data register	
}



/*********************************************************************************************
Function name   : TG12864H3_Init
Author 					: Grant Phillips
Date Modified   : 02/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Initializes the TG12864H3 lcd module

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void TG12864H3_Init(void)
{
	unsigned char i,j;
	GPIO_InitTypeDef 	GPIO_InitStructure;	
	SPI_InitTypeDef 	SPI_InitStruct;
	
	/* //////////////////////////////////////////////////////////////
	Do the proper GPIO initialization for the output port pins here
	////////////////////////////////////////////////////////////// */
	RCC_AHB1PeriphClockCmd (RCC_AHB1Periph_GPIOB | RCC_AHB1Periph_GPIOD | RCC_AHB1Periph_GPIOE, ENABLE);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;									
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOD, &GPIO_InitStructure);
	
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;									
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOE, &GPIO_InitStructure);

	/* Configure GPIO pins to SPI alternate function */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13 | GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource13, GPIO_AF_SPI2);
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource15, GPIO_AF_SPI2);
	
	/* Enable SPI2 peripheral clock */
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);
	
	/* configure SPI2 in Mode 0 */
	SPI_InitStruct.SPI_Direction = SPI_Direction_2Lines_FullDuplex; 	// set to full duplex mode, seperate MOSI and MISO lines
	SPI_InitStruct.SPI_Mode = SPI_Mode_Master;     										// transmit in master mode, NSS pin has to be always high
	SPI_InitStruct.SPI_DataSize = SPI_DataSize_8b; 										// one packet of data is 8 bits wide
	SPI_InitStruct.SPI_CPOL = SPI_CPOL_Low;        										// clock is low when idle
	SPI_InitStruct.SPI_CPHA = SPI_CPHA_1Edge;      										// data sampled at first edge
	SPI_InitStruct.SPI_NSS = SPI_NSS_Soft | SPI_NSSInternalSoft_Set; 	// set the NSS management to internal and pull internal NSS high
	SPI_InitStruct.SPI_BaudRatePrescaler = SPI_BaudRatePrescaler_32; 	// SPI frequency is APB2 frequency / 32
	SPI_InitStruct.SPI_FirstBit = SPI_FirstBit_MSB;										// data is transmitted MSB first
	SPI_Init(SPI2, &SPI_InitStruct); 
	
	/* Enable SPI2 */
	SPI_Cmd(SPI2, ENABLE);
	/////////////////////////////////////////////////////////////////
	
	lcd_cs_0;
	lcd_rst_0;
	TG12864H3_Delay(lcd_init_delay);
	lcd_rst_1;


	TG12864H3_Command(CMD_SET_BIAS_7);
  TG12864H3_Command(CMD_SET_ADC_NORMAL);						//ADC=0 Normal
  TG12864H3_Command(CMD_SET_DISP_NORMAL);
  TG12864H3_Command(CMD_SET_COM_REVERSE);
  TG12864H3_Command(CMD_SET_DISP_START_LINE);
  TG12864H3_Command(CMD_SET_POWER_CONTROL | 0x07);	//Power Control VF, VR, VC
  TG12864H3_Command(CMD_SET_RESISTOR_RATIO | 0x6);
  TG12864H3_Command(CMD_DISPLAY_ON);
  TG12864H3_Command(CMD_SET_ALLPTS_NORMAL);
  TG12864H3_SetContrast(lcd_contrast);

	for(i=0;i<128;i++)
		for(j=0;j<8;j++)
			TG12864H3_DisplayRAM[i][j]=0x00;

	lcd_cs_1;
	
	TG12864H3_ClrScr(0);
}



/*********************************************************************************************
Function name   : TG12864H3_Delay
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Creates a delay which is simply a code loop which is independent from any
									hardware timers.

Special Note(s) : NONE

Parameters			: del		-	16-bit value to represent the delay cycles
Return value		: NONE
*********************************************************************************************/
void TG12864H3_Delay(unsigned int del)
{
	unsigned int i=0;

	while(i<del)
		i++;
}



/*********************************************************************************************
Function name   : TG12864H3_Command
Author 					: Grant Phillips
Date Modified   : 02/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Writes a COMMAND to the TG12864H3 lcd module

Special Note(s) : NONE

Parameters			: cmd		-	The byte of command to write to the lcd
Return value		: NONE
*********************************************************************************************/
void TG12864H3_Command(unsigned char cmd)
{
	lcd_a0_0;
	TG1286H3_SPI2_send(cmd);
}



/*********************************************************************************************
Function name   : TG12864H3_WriteData
Author 					: Grant Phillips
Date Modified   : 02/10/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Writes DATA to the TG12864H3 lcd module

Special Note(s) : NONE

Parameters			: dat		-	The byte of data to write to the lcd
Return value		: NONE
*********************************************************************************************/
void TG12864H3_WriteData(unsigned char dat)
{
	lcd_a0_1;
	TG1286H3_SPI2_send(dat);
}



/*********************************************************************************************
Function name   : TG12864H3_SetPage
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Set the display RAM page address

Special Note(s) : NONE

Parameters			: Page		-	4-bit value representing the page address (0 - 7)
Return value		: NONE
*********************************************************************************************/
void TG12864H3_SetPage(unsigned char Page)
{
	Page = Page & 0x0f;
	Page = Page | 0xb0;
	TG12864H3_Command(Page);
}



/*********************************************************************************************
Function name   : TG12864H3_SetColumn
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Set the column address counter

Special Note(s) : NONE

Parameters			: Page		-	byte value representing the column (0 - 127)
Return value		: NONE
*********************************************************************************************/
void TG12864H3_SetColumn(unsigned char Col)
{
	unsigned char temp;

	Col = Col + 1;
	
	temp = Col;
	Col = Col & 0x0f;
	Col = Col | 0x00;
	TG12864H3_Command(Col);
	temp = temp >> 4;
	Col = temp & 0x0f;
	Col = Col | 0x10;
	TG12864H3_Command(Col);
}



/*********************************************************************************************
Function name   : TG12864H3_ClrScr
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Clears the display by clearing or filling all the pixels

Special Note(s) : NONE

Parameters			: color		-	bit value; clear pixels(0) or fill pixels(1)
Return value		: NONE
*********************************************************************************************/
void TG12864H3_ClrScr(unsigned char color)
{
	unsigned char i, j, x=0;

	lcd_cs_0;
	for(i=0; i<8; i++)
	{
		x=0;
		TG12864H3_SetColumn(0);
		TG12864H3_SetPage(i);
		for(j=0; j<128; j++)
			if(color==0)
			{
				TG12864H3_WriteData(0x00);
				TG12864H3_DisplayRAM[x][i] = 0x00;
				x++;
			}
			else
			{
				TG12864H3_WriteData(0xff);
				TG12864H3_DisplayRAM[x][i] = 0xff;
				x++;
			}
	}
	lcd_cs_1;
}



/*********************************************************************************************
Function name   : TG12864H3_SetContrast
Author 					: Grant Phillips
Date Modified   : 10/02/2014
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Set the contrast of the LCD

Special Note(s) : NONE

Parameters			: value		-	6-bit value representing the brightness (0 - 63)
Return value		: NONE
*********************************************************************************************/
void TG12864H3_SetContrast(unsigned char value) 
{    
   TG12864H3_Command(CMD_SET_VOLUME_FIRST);    
   TG12864H3_Command(CMD_SET_VOLUME_SECOND | (value & 0x3f));
}



/*********************************************************************************************
Function name   : TG12864H3_PowerSave
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Puts the lcd in power save mode; display off and backlight off (if BLA pin is
									#defined)

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void TG12864H3_PowerSave(void)
{
#ifdef lcd_bla_1
	lcd_bla_0;
#endif
	lcd_cs_0;
	TG12864H3_Command(0xae);
	TG12864H3_Command(0xa5);
	lcd_cs_1;
}



/*********************************************************************************************
Function name   : TG12864H3_PowerActive
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : ARM GCC, Keil ARM-MDK

Description			: Puts the lcd in normal power mode; display on and backlight on (if BLA pin
									is #defined)

Special Note(s) : NONE

Parameters			: NONE
Return value		: NONE
*********************************************************************************************/
void TG12864H3_PowerActive(void)
{
#ifdef lcd_bla_1
	lcd_bla_1;
#endif
	lcd_cs_0;
	TG12864H3_Command(0xa4);
	TG12864H3_Command(0xaf);
	lcd_cs_1;
}


 
/*********************************************************************************************
Function name   : TG12864H3_DrawPixel
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Draws a pixel at the specified xy position

Special Note(s) : NONE

Parameters			: x				-	x position (0 - 127)
									y				-	y position (0 - 63)
									color		-	color of the pixel; cleared(0), filled(1), or toggle(2)
Return value		: NONE
*********************************************************************************************/
void TG12864H3_DrawPixel(unsigned char x, unsigned char y, unsigned char color)
{
	unsigned char page=0, temp1, temp2;

	if(y <= 7)
		page = 4;
	else if(y <= 15)
		page = 5;
	else if(y <= 23)
		page = 6;
	else if(y <= 31)
		page = 7;
	else if(y <= 39)
		page = 0;
	else if(y <= 47)
		page = 1;
	else if(y <= 55)
		page = 2;
	else if(y <= 63)
		page = 3;
	
	y = y & 0x07;
	temp1 = TG12864H3_DisplayRAM[x][page];
	temp2 = 1 << y;
	switch(color)
	{
		case 0:	temp1 = temp1 & ~temp2;
      			break;
		case 1: temp1 = temp1 | temp2;
				break;
		case 2: temp1 = temp1 ^ temp2;
				break;
	}
	TG12864H3_DisplayRAM[x][page] = temp1;
	lcd_cs_0;
	TG12864H3_SetColumn(x);
	TG12864H3_SetPage(page);
	TG12864H3_WriteData((unsigned char)(temp1));
	lcd_cs_1;
}



/*********************************************************************************************
Function name   : TG12864H3_DrawLine
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Draws a line from x1,y1 to x2,y2 in color

Special Note(s) : NONE

Parameters			: x1			-	x1 position (0 - 127)
									y1			-	y1 position (0 - 63)
									x2			-	x2 position (0 - 127)
									y2			-	y2 position (0 - 63)
									color		-	color of the pixel; cleared(0), filled(1), or toggle(2)
Return value		: NONE
*********************************************************************************************/
void TG12864H3_DrawLine(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, unsigned int color)
{
   int        dy, dx;
   signed char  addx=1, addy=1;
   signed int P, diff;

   int i=0;
	
   dx = abs((signed int)(x2 - x1));
   dy = abs((signed int)(y2 - y1));
	
   if(x1 > x2)
      addx = -1;
   if(y1 > y2) 
      addy = -1;

   if(dx >= dy)
   {
      dy *= 2;
      P = dy - dx;
      diff = P - dx;

      for(; i<=dx; ++i)
      {
         TG12864H3_DrawPixel(x1, y1, color);

         if(P < 0)
         {
            P  += dy;
            x1 += addx;
         }
         else
         {
            P  += diff;
            x1 += addx;
            y1 += addy;
         }
      }
   }
   else
   {
      dx *= 2;
      P = dx - dy;
      diff = P - dy;

      for(; i<=dy; ++i)
      {
    	  TG12864H3_DrawPixel(x1, y1, color);

         if(P < 0)
         {
            P  += dx;
            y1 += addy;
         }
         else
         {
            P  += diff;
            x1 += addx;
            y1 += addy;
         }
      }
   }
}



/*********************************************************************************************
Function name   : TG12864H3_DrawCircle
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Draws a circle at x,y.  The color can set and the filled or not filled 
									option can be selected.

Special Note(s) : NONE

Parameters			: x				-	x position (0 - 127)
									y				-	y position (0 - 63)
									radius	-	radius of the circle (0 - 127)
									fill		-	circle is filled (=1) or not filled (=0)
									color		-	color of the pixel; cleared(0), filled(1)
Return value		: NONE
*********************************************************************************************/
void TG12864H3_DrawCircle(unsigned char x, unsigned char y, unsigned char radius, unsigned char fill, unsigned char color)
{
   signed char a, b, P;

   a = 0;
   b = radius;
   P = 1 - radius;

   do
   {
      if(fill)
      {
         TG12864H3_DrawLine(x-a, y+b, x+a, y+b, color);
         TG12864H3_DrawLine(x-a, y-b, x+a, y-b, color);
         TG12864H3_DrawLine(x-b, y+a, x+b, y+a, color);
         TG12864H3_DrawLine(x-b, y-a, x+b, y-a, color);
      }
      else
      {
         TG12864H3_DrawPixel(a+x, b+y, color);
         TG12864H3_DrawPixel(b+x, a+y, color);
         TG12864H3_DrawPixel(x-a, b+y, color);
         TG12864H3_DrawPixel(x-b, a+y, color);
         TG12864H3_DrawPixel(b+x, y-a, color);
         TG12864H3_DrawPixel(a+x, y-b, color);
         TG12864H3_DrawPixel(x-a, y-b, color);
         TG12864H3_DrawPixel(x-b, y-a, color);
      }

      if(P < 0)
         P += 3 + 2 * a++;
      else
         P += 5 + 2 * (a++ - b--);
    } while(a <= b);
}



/*********************************************************************************************
Function name   : TG12864H3_GotoXY
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Changes the position of the current character cursor from where the next 
									characters will be printed.

Special Note(s) : NONE

Parameters			: x				-	column position (0 - 20)
									y				-	row position (0 - 7)

Return value		: NONE
*********************************************************************************************/
void TG12864H3_GotoXY(unsigned char x, unsigned char y)
{
	//if((x>=0)&&(x<=20))
  if(x<=20)
		lcd_char_x = x;
	else
		lcd_char_x = 0;
	//if((y>=0)&&(y<=7))
	if(y<=7)
		lcd_char_y = y;
	else
		lcd_char_y = 0;
}



/*********************************************************************************************
Function name   : TG12864H3_PutStr
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Prints a string at the current character cursor position with a selected
									color.

Special Note(s) : NONE

Parameters			: str			-	string (char array) to print
									color		-	color of the char in the string; normal(1), inverted(0)

Return value		: NONE
*********************************************************************************************/
void TG12864H3_PutStr(char *str, unsigned char color)
{
	unsigned int i=0;

	do
	{
		TG12864H3_PutChar(str[i], color);
		i++;
	}while(str[i]!='\0');

}



/*********************************************************************************************
Function name   : TG12864H3_PutChar
Author 					: Grant Phillips
Date Modified   : 02/10/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Prints a character at the current character cursor position with a selected
									color.

Special Note(s) : NONE

Parameters			: c				-	character to print
									color		-	color of the character; normal(1), inverted(0)

Return value		: NONE
*********************************************************************************************/
void TG12864H3_PutChar(unsigned char c, unsigned char color)
{
	unsigned char x, page=0;

	x = lcd_char_x * 6 + 1;
	
	//page = lcd_char_y;
	if(lcd_char_y == 0)
		page = 4;
	else if(lcd_char_y == 1)
		page = 5;
	else if(lcd_char_y == 2)
		page = 6;
	else if(lcd_char_y == 3)
		page = 7;
	else if(lcd_char_y == 4)
		page = 0;
	else if(lcd_char_y == 5)
		page = 1;
	else if(lcd_char_y == 6)
		page = 2;
	else if(lcd_char_y == 7)
		page = 3;

	TG12864H3_DrawChar(c, x, page, color);
	if(lcd_char_x == 20)
	{
		lcd_char_x=0;
		if(lcd_char_y==7)
			lcd_char_y=0;
		else
			lcd_char_y++;
	}
	else
		lcd_char_x++;
}


/*********************************************************************************************
Function name   : TG12864H3_WriteCharCol
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Used by TG12864H3_DrawChar only updates the column and page numbers.
*********************************************************************************************/
void TG12864H3_WriteCharCol(unsigned char v, unsigned char x, unsigned char page, unsigned char color)
{
	if(color==0)
	{
		TG12864H3_WriteData(~v);
		TG12864H3_DisplayRAM[x][page] = ~v;
	}
	else
	{
		TG12864H3_WriteData(v);
		TG12864H3_DisplayRAM[x][page] = v;
	}
}



/*********************************************************************************************
Function name   : TG12864H3_DrawChar
Author 					: Grant Phillips
Date Modified   : 25/08/2013
Compiler        : Keil ARM-MDK (uVision V4.70.0.0)

Description			: Used by TG12864H3_PutChar() only and draws the specified character at byte 
									level using a set font.
*********************************************************************************************/
void TG12864H3_DrawChar(unsigned char c, unsigned char x, unsigned char page, unsigned char color)
{
	lcd_cs_0;
	TG12864H3_SetColumn(x);
	TG12864H3_SetPage(page);

	switch(c)
	{
		case 32:	// 'SPACE'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x00, x+1, page, color);
			TG12864H3_WriteCharCol(0x00, x+2, page, color);
			TG12864H3_WriteCharCol(0x00, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 33:	// '!'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x00, x+1, page, color);
			TG12864H3_WriteCharCol(0x9e, x+2, page, color);
			TG12864H3_WriteCharCol(0x00, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 34:	// '"'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x0e, x+1, page, color);
			TG12864H3_WriteCharCol(0x00, x+2, page, color);
			TG12864H3_WriteCharCol(0x0e, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 35:	// '#'
		{
			TG12864H3_WriteCharCol(0x28, x, page, color);
			TG12864H3_WriteCharCol(0xfe, x+1, page, color);
			TG12864H3_WriteCharCol(0x28, x+2, page, color);
			TG12864H3_WriteCharCol(0xfe, x+3, page, color);
			TG12864H3_WriteCharCol(0x28, x+4, page, color);
		}break;

		case 36:	// '$'
		{
			TG12864H3_WriteCharCol(0x48, x, page, color);
			TG12864H3_WriteCharCol(0x54, x+1, page, color);
			TG12864H3_WriteCharCol(0xfe, x+2, page, color);
			TG12864H3_WriteCharCol(0x54, x+3, page, color);
			TG12864H3_WriteCharCol(0x24, x+4, page, color);
		}break;

		case 37:	// '%'
		{
			TG12864H3_WriteCharCol(0x46, x, page, color);
			TG12864H3_WriteCharCol(0x26, x+1, page, color);
			TG12864H3_WriteCharCol(0x10, x+2, page, color);
			TG12864H3_WriteCharCol(0xc8, x+3, page, color);
			TG12864H3_WriteCharCol(0xc4, x+4, page, color);
		}break;

		case 38:	// '&'
		{
			TG12864H3_WriteCharCol(0x6c, x, page, color);
			TG12864H3_WriteCharCol(0x92, x+1, page, color);
			TG12864H3_WriteCharCol(0xaa, x+2, page, color);
			TG12864H3_WriteCharCol(0x44, x+3, page, color);
			TG12864H3_WriteCharCol(0xa0, x+4, page, color);
		}break;

		case 39:	// '''
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x0a, x+1, page, color);
			TG12864H3_WriteCharCol(0x06, x+2, page, color);
			TG12864H3_WriteCharCol(0x00, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 40:	// '('
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x38, x+1, page, color);
			TG12864H3_WriteCharCol(0x44, x+2, page, color);
			TG12864H3_WriteCharCol(0x82, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 41:	// ')'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x82, x+1, page, color);
			TG12864H3_WriteCharCol(0x44, x+2, page, color);
			TG12864H3_WriteCharCol(0x38, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 42:	// '*'
		{
			TG12864H3_WriteCharCol(0x28, x, page, color);
			TG12864H3_WriteCharCol(0x10, x+1, page, color);
			TG12864H3_WriteCharCol(0x7c, x+2, page, color);
			TG12864H3_WriteCharCol(0x10, x+3, page, color);
			TG12864H3_WriteCharCol(0x28, x+4, page, color);
		}break;

		case 43:	// '+'
		{
			TG12864H3_WriteCharCol(0x10, x, page, color);
			TG12864H3_WriteCharCol(0x10, x+1, page, color);
			TG12864H3_WriteCharCol(0x7c, x+2, page, color);
			TG12864H3_WriteCharCol(0x10, x+3, page, color);
			TG12864H3_WriteCharCol(0x10, x+4, page, color);
		}break;

		case 44:	// ','
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0xa0, x+1, page, color);
			TG12864H3_WriteCharCol(0x60, x+2, page, color);
			TG12864H3_WriteCharCol(0x00, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 45:	// '-'
		{
			TG12864H3_WriteCharCol(0x10, x, page, color);
			TG12864H3_WriteCharCol(0x10, x+1, page, color);
			TG12864H3_WriteCharCol(0x10, x+2, page, color);
			TG12864H3_WriteCharCol(0x10, x+3, page, color);
			TG12864H3_WriteCharCol(0x10, x+4, page, color);
		}break;

		case 46:	// '.'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0xc0, x+1, page, color);
			TG12864H3_WriteCharCol(0xc0, x+2, page, color);
			TG12864H3_WriteCharCol(0x00, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 47:	// '/'
		{
			TG12864H3_WriteCharCol(0x40, x, page, color);
			TG12864H3_WriteCharCol(0x20, x+1, page, color);
			TG12864H3_WriteCharCol(0x10, x+2, page, color);
			TG12864H3_WriteCharCol(0x08, x+3, page, color);
			TG12864H3_WriteCharCol(0x04, x+4, page, color);
		}break;

		case 48:	// '0'
		{
			TG12864H3_WriteCharCol(0x7c, x, page, color);
			TG12864H3_WriteCharCol(0xa2, x+1, page, color);
			TG12864H3_WriteCharCol(0x92, x+2, page, color);
			TG12864H3_WriteCharCol(0x8a, x+3, page, color);
			TG12864H3_WriteCharCol(0x7c, x+4, page, color);
		}break;

		case 49:	// '1'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x84, x+1, page, color);
			TG12864H3_WriteCharCol(0xfe, x+2, page, color);
			TG12864H3_WriteCharCol(0x80, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 50:	// '2'
		{
			TG12864H3_WriteCharCol(0x84, x, page, color);
			TG12864H3_WriteCharCol(0xc2, x+1, page, color);
			TG12864H3_WriteCharCol(0xa2, x+2, page, color);
			TG12864H3_WriteCharCol(0x92, x+3, page, color);
			TG12864H3_WriteCharCol(0x8c, x+4, page, color);
		}break;

		case 51:	// '3'
		{
			TG12864H3_WriteCharCol(0x42, x, page, color);
			TG12864H3_WriteCharCol(0x82, x+1, page, color);
			TG12864H3_WriteCharCol(0x8a, x+2, page, color);
			TG12864H3_WriteCharCol(0x96, x+3, page, color);
			TG12864H3_WriteCharCol(0x62, x+4, page, color);
		}break;

		case 52:	// '4'
		{
			TG12864H3_WriteCharCol(0x30, x, page, color);
			TG12864H3_WriteCharCol(0x28, x+1, page, color);
			TG12864H3_WriteCharCol(0x24, x+2, page, color);
			TG12864H3_WriteCharCol(0xfe, x+3, page, color);
			TG12864H3_WriteCharCol(0x20, x+4, page, color);
		}break;

		case 53:	// '5'
		{
			TG12864H3_WriteCharCol(0x4e, x, page, color);
			TG12864H3_WriteCharCol(0x8a, x+1, page, color);
			TG12864H3_WriteCharCol(0x8a, x+2, page, color);
			TG12864H3_WriteCharCol(0x8a, x+3, page, color);
			TG12864H3_WriteCharCol(0x72, x+4, page, color);
		}break;

		case 54:	// '6'
		{
			TG12864H3_WriteCharCol(0x78, x, page, color);
			TG12864H3_WriteCharCol(0x94, x+1, page, color);
			TG12864H3_WriteCharCol(0x92, x+2, page, color);
			TG12864H3_WriteCharCol(0x92, x+3, page, color);
			TG12864H3_WriteCharCol(0x60, x+4, page, color);
		}break;

		case 55:	// '7'
		{
			TG12864H3_WriteCharCol(0x02, x, page, color);
			TG12864H3_WriteCharCol(0xe2, x+1, page, color);
			TG12864H3_WriteCharCol(0x12, x+2, page, color);
			TG12864H3_WriteCharCol(0x0a, x+3, page, color);
			TG12864H3_WriteCharCol(0x06, x+4, page, color);
		}break;

		case 56:	// '8'
		{
			TG12864H3_WriteCharCol(0x6c, x, page, color);
			TG12864H3_WriteCharCol(0x92, x+1, page, color);
			TG12864H3_WriteCharCol(0x92, x+2, page, color);
			TG12864H3_WriteCharCol(0x92, x+3, page, color);
			TG12864H3_WriteCharCol(0x6c, x+4, page, color);
		}break;

		case 57:	// '9'
		{
			TG12864H3_WriteCharCol(0x0c, x, page, color);
			TG12864H3_WriteCharCol(0x92, x+1, page, color);
			TG12864H3_WriteCharCol(0x92, x+2, page, color);
			TG12864H3_WriteCharCol(0x52, x+3, page, color);
			TG12864H3_WriteCharCol(0x3c, x+4, page, color);
		}break;

		case 58:	// ':'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x6c, x+1, page, color);
			TG12864H3_WriteCharCol(0x6c, x+2, page, color);
			TG12864H3_WriteCharCol(0x00, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 59:	// ';'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0xac, x+1, page, color);
			TG12864H3_WriteCharCol(0x6c, x+2, page, color);
			TG12864H3_WriteCharCol(0x00, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 60:	// '<'
		{
			TG12864H3_WriteCharCol(0x10, x, page, color);
			TG12864H3_WriteCharCol(0x28, x+1, page, color);
			TG12864H3_WriteCharCol(0x44, x+2, page, color);
			TG12864H3_WriteCharCol(0x82, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 61:	// '='
		{
			TG12864H3_WriteCharCol(0x28, x, page, color);
			TG12864H3_WriteCharCol(0x28, x+1, page, color);
			TG12864H3_WriteCharCol(0x28, x+2, page, color);
			TG12864H3_WriteCharCol(0x28, x+3, page, color);
			TG12864H3_WriteCharCol(0x28, x+4, page, color);
		}break;

		case 62:	// '>'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x82, x+1, page, color);
			TG12864H3_WriteCharCol(0x44, x+2, page, color);
			TG12864H3_WriteCharCol(0x28, x+3, page, color);
			TG12864H3_WriteCharCol(0x10, x+4, page, color);
		}break;

		case 63:	// '?'
		{
			TG12864H3_WriteCharCol(0x04, x, page, color);
			TG12864H3_WriteCharCol(0x02, x+1, page, color);
			TG12864H3_WriteCharCol(0xa2, x+2, page, color);
			TG12864H3_WriteCharCol(0x12, x+3, page, color);
			TG12864H3_WriteCharCol(0x0c, x+4, page, color);
		}break;

		case 64:	// '@'
		{
			TG12864H3_WriteCharCol(0x64, x, page, color);
			TG12864H3_WriteCharCol(0x92, x+1, page, color);
			TG12864H3_WriteCharCol(0xf2, x+2, page, color);
			TG12864H3_WriteCharCol(0x82, x+3, page, color);
			TG12864H3_WriteCharCol(0x7c, x+4, page, color);
		}break;

		case 65:	// 'A'
		{
			TG12864H3_WriteCharCol(0xfc, x, page, color);
			TG12864H3_WriteCharCol(0x22, x+1, page, color);
			TG12864H3_WriteCharCol(0x22, x+2, page, color);
			TG12864H3_WriteCharCol(0x22, x+3, page, color);
			TG12864H3_WriteCharCol(0xfc, x+4, page, color);
		}break;

		case 66:	// 'B'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x92, x+1, page, color);
			TG12864H3_WriteCharCol(0x92, x+2, page, color);
			TG12864H3_WriteCharCol(0x92, x+3, page, color);
			TG12864H3_WriteCharCol(0x6c, x+4, page, color);
		}break;

		case 67:	// 'C'
		{
			TG12864H3_WriteCharCol(0x7c, x, page, color);
			TG12864H3_WriteCharCol(0x82, x+1, page, color);
			TG12864H3_WriteCharCol(0x82, x+2, page, color);
			TG12864H3_WriteCharCol(0x82, x+3, page, color);
			TG12864H3_WriteCharCol(0x44, x+4, page, color);
		}break;

		case 68:	// 'D'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x82, x+1, page, color);
			TG12864H3_WriteCharCol(0x82, x+2, page, color);
			TG12864H3_WriteCharCol(0x44, x+3, page, color);
			TG12864H3_WriteCharCol(0x38, x+4, page, color);
		}break;

		case 69:	// 'E'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x92, x+1, page, color);
			TG12864H3_WriteCharCol(0x92, x+2, page, color);
			TG12864H3_WriteCharCol(0x92, x+3, page, color);
			TG12864H3_WriteCharCol(0x82, x+4, page, color);
		}break;

		case 70:	// 'F'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x12, x+1, page, color);
			TG12864H3_WriteCharCol(0x12, x+2, page, color);
			TG12864H3_WriteCharCol(0x12, x+3, page, color);
			TG12864H3_WriteCharCol(0x02, x+4, page, color);
		}break;

		case 71:	// 'G'
		{
			TG12864H3_WriteCharCol(0x7c, x, page, color);
			TG12864H3_WriteCharCol(0x82, x+1, page, color);
			TG12864H3_WriteCharCol(0x92, x+2, page, color);
			TG12864H3_WriteCharCol(0x92, x+3, page, color);
			TG12864H3_WriteCharCol(0xf4, x+4, page, color);
		}break;

		case 72:	// 'H'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x10, x+1, page, color);
			TG12864H3_WriteCharCol(0x10, x+2, page, color);
			TG12864H3_WriteCharCol(0x10, x+3, page, color);
			TG12864H3_WriteCharCol(0xfe, x+4, page, color);
		}break;

		case 73:	// 'I'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x82, x+1, page, color);
			TG12864H3_WriteCharCol(0xfe, x+2, page, color);
			TG12864H3_WriteCharCol(0x82, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 74:	// 'J'
		{
			TG12864H3_WriteCharCol(0x40, x, page, color);
			TG12864H3_WriteCharCol(0x80, x+1, page, color);
			TG12864H3_WriteCharCol(0x82, x+2, page, color);
			TG12864H3_WriteCharCol(0x7e, x+3, page, color);
			TG12864H3_WriteCharCol(0x02, x+4, page, color);
		}break;

		case 75:	// 'K'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x10, x+1, page, color);
			TG12864H3_WriteCharCol(0x28, x+2, page, color);
			TG12864H3_WriteCharCol(0x44, x+3, page, color);
			TG12864H3_WriteCharCol(0x82, x+4, page, color);
		}break;

		case 76:	// 'L'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x80, x+1, page, color);
			TG12864H3_WriteCharCol(0x80, x+2, page, color);
			TG12864H3_WriteCharCol(0x80, x+3, page, color);
			TG12864H3_WriteCharCol(0x80, x+4, page, color);
		}break;

		case 77:	// 'M'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x04, x+1, page, color);
			TG12864H3_WriteCharCol(0x18, x+2, page, color);
			TG12864H3_WriteCharCol(0x04, x+3, page, color);
			TG12864H3_WriteCharCol(0xfe, x+4, page, color);
		}break;

		case 78:	// 'N'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x08, x+1, page, color);
			TG12864H3_WriteCharCol(0x10, x+2, page, color);
			TG12864H3_WriteCharCol(0x20, x+3, page, color);
			TG12864H3_WriteCharCol(0xfe, x+4, page, color);
		}break;

		case 79:	// 'O'
		{
			TG12864H3_WriteCharCol(0x7c, x, page, color);
			TG12864H3_WriteCharCol(0x82, x+1, page, color);
			TG12864H3_WriteCharCol(0x82, x+2, page, color);
			TG12864H3_WriteCharCol(0x82, x+3, page, color);
			TG12864H3_WriteCharCol(0x7c, x+4, page, color);
		}break;

		case 80:	// 'P'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x12, x+1, page, color);
			TG12864H3_WriteCharCol(0x12, x+2, page, color);
			TG12864H3_WriteCharCol(0x12, x+3, page, color);
			TG12864H3_WriteCharCol(0x0c, x+4, page, color);
		}break;

		case 81:	// 'Q'
		{
			TG12864H3_WriteCharCol(0x7c, x, page, color);
			TG12864H3_WriteCharCol(0x82, x+1, page, color);
			TG12864H3_WriteCharCol(0xa2, x+2, page, color);
			TG12864H3_WriteCharCol(0x42, x+3, page, color);
			TG12864H3_WriteCharCol(0xbc, x+4, page, color);
		}break;

		case 82:	// 'R'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x12, x+1, page, color);
			TG12864H3_WriteCharCol(0x32, x+2, page, color);
			TG12864H3_WriteCharCol(0x52, x+3, page, color);
			TG12864H3_WriteCharCol(0x8c, x+4, page, color);
		}break;

		case 83:	// 'S'
		{
			TG12864H3_WriteCharCol(0x8c, x, page, color);
			TG12864H3_WriteCharCol(0x92, x+1, page, color);
			TG12864H3_WriteCharCol(0x92, x+2, page, color);
			TG12864H3_WriteCharCol(0x92, x+3, page, color);
			TG12864H3_WriteCharCol(0x62, x+4, page, color);
		}break;

		case 84:	// 'T'
		{
			TG12864H3_WriteCharCol(0x02, x, page, color);
			TG12864H3_WriteCharCol(0x02, x+1, page, color);
			TG12864H3_WriteCharCol(0xfe, x+2, page, color);
			TG12864H3_WriteCharCol(0x02, x+3, page, color);
			TG12864H3_WriteCharCol(0x02, x+4, page, color);
		}break;

		case 85:	// 'U'
		{
			TG12864H3_WriteCharCol(0x7e, x, page, color);
			TG12864H3_WriteCharCol(0x80, x+1, page, color);
			TG12864H3_WriteCharCol(0x80, x+2, page, color);
			TG12864H3_WriteCharCol(0x80, x+3, page, color);
			TG12864H3_WriteCharCol(0x7e, x+4, page, color);
		}break;

		case 86:	// 'V'
		{
			TG12864H3_WriteCharCol(0x3e, x, page, color);
			TG12864H3_WriteCharCol(0x40, x+1, page, color);
			TG12864H3_WriteCharCol(0x80, x+2, page, color);
			TG12864H3_WriteCharCol(0x40, x+3, page, color);
			TG12864H3_WriteCharCol(0x3e, x+4, page, color);
		}break;

		case 87:	// 'W'
		{
			TG12864H3_WriteCharCol(0x7e, x, page, color);
			TG12864H3_WriteCharCol(0x80, x+1, page, color);
			TG12864H3_WriteCharCol(0x70, x+2, page, color);
			TG12864H3_WriteCharCol(0x80, x+3, page, color);
			TG12864H3_WriteCharCol(0x7e, x+4, page, color);
		}break;

		case 88:	// 'X'
		{
			TG12864H3_WriteCharCol(0xc6, x, page, color);
			TG12864H3_WriteCharCol(0x28, x+1, page, color);
			TG12864H3_WriteCharCol(0x10, x+2, page, color);
			TG12864H3_WriteCharCol(0x28, x+3, page, color);
			TG12864H3_WriteCharCol(0xc6, x+4, page, color);
		}break;

		case 89:	// 'Y'
		{
			TG12864H3_WriteCharCol(0x0e, x, page, color);
			TG12864H3_WriteCharCol(0x10, x+1, page, color);
			TG12864H3_WriteCharCol(0xe0, x+2, page, color);
			TG12864H3_WriteCharCol(0x10, x+3, page, color);
			TG12864H3_WriteCharCol(0x0e, x+4, page, color);
		}break;

		case 90:	// 'Z'
		{
			TG12864H3_WriteCharCol(0xc2, x, page, color);
			TG12864H3_WriteCharCol(0xa2, x+1, page, color);
			TG12864H3_WriteCharCol(0x92, x+2, page, color);
			TG12864H3_WriteCharCol(0x8a, x+3, page, color);
			TG12864H3_WriteCharCol(0x86, x+4, page, color);
		}break;

		case 91:	// '['
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0xfe, x+1, page, color);
			TG12864H3_WriteCharCol(0x82, x+2, page, color);
			TG12864H3_WriteCharCol(0x82, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 92:	// '\'
		{
			TG12864H3_WriteCharCol(0x04, x, page, color);
			TG12864H3_WriteCharCol(0x08, x+1, page, color);
			TG12864H3_WriteCharCol(0x10, x+2, page, color);
			TG12864H3_WriteCharCol(0x20, x+3, page, color);
			TG12864H3_WriteCharCol(0x40, x+4, page, color);
		}break;

		case 93:	// ']'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x82, x+1, page, color);
			TG12864H3_WriteCharCol(0x82, x+2, page, color);
			TG12864H3_WriteCharCol(0xfe, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 94:	// '^'
		{
			TG12864H3_WriteCharCol(0x08, x, page, color);
			TG12864H3_WriteCharCol(0x04, x+1, page, color);
			TG12864H3_WriteCharCol(0x02, x+2, page, color);
			TG12864H3_WriteCharCol(0x04, x+3, page, color);
			TG12864H3_WriteCharCol(0x08, x+4, page, color);
		}break;

		case 95:	// '_'
		{
			TG12864H3_WriteCharCol(0x80, x, page, color);
			TG12864H3_WriteCharCol(0x80, x+1, page, color);
			TG12864H3_WriteCharCol(0x80, x+2, page, color);
			TG12864H3_WriteCharCol(0x80, x+3, page, color);
			TG12864H3_WriteCharCol(0x80, x+4, page, color);
		}break;

		case 96:	// '`'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x02, x+1, page, color);
			TG12864H3_WriteCharCol(0x04, x+2, page, color);
			TG12864H3_WriteCharCol(0x08, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 97:	// 'a'
		{
			TG12864H3_WriteCharCol(0x40, x, page, color);
			TG12864H3_WriteCharCol(0xa8, x+1, page, color);
			TG12864H3_WriteCharCol(0xa8, x+2, page, color);
			TG12864H3_WriteCharCol(0xa8, x+3, page, color);
			TG12864H3_WriteCharCol(0xf0, x+4, page, color);
		}break;

		case 98:	// 'b'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x90, x+1, page, color);
			TG12864H3_WriteCharCol(0x88, x+2, page, color);
			TG12864H3_WriteCharCol(0x88, x+3, page, color);
			TG12864H3_WriteCharCol(0x70, x+4, page, color);
		}break;

		case 99:	// 'c'
		{
			TG12864H3_WriteCharCol(0x70, x, page, color);
			TG12864H3_WriteCharCol(0x88, x+1, page, color);
			TG12864H3_WriteCharCol(0x88, x+2, page, color);
			TG12864H3_WriteCharCol(0x88, x+3, page, color);
			TG12864H3_WriteCharCol(0x40, x+4, page, color);
		}break;

		case 100:	// 'd'
		{
			TG12864H3_WriteCharCol(0x70, x, page, color);
			TG12864H3_WriteCharCol(0x88, x+1, page, color);
			TG12864H3_WriteCharCol(0x88, x+2, page, color);
			TG12864H3_WriteCharCol(0x90, x+3, page, color);
			TG12864H3_WriteCharCol(0xfe, x+4, page, color);
		}break;

		case 101:	// 'e'
		{
			TG12864H3_WriteCharCol(0x70, x, page, color);
			TG12864H3_WriteCharCol(0xa8, x+1, page, color);
			TG12864H3_WriteCharCol(0xa8, x+2, page, color);
			TG12864H3_WriteCharCol(0xa8, x+3, page, color);
			TG12864H3_WriteCharCol(0x30, x+4, page, color);
		}break;

		case 102:	// 'f'
		{
			TG12864H3_WriteCharCol(0x10, x, page, color);
			TG12864H3_WriteCharCol(0xfc, x+1, page, color);
			TG12864H3_WriteCharCol(0x12, x+2, page, color);
			TG12864H3_WriteCharCol(0x02, x+3, page, color);
			TG12864H3_WriteCharCol(0x04, x+4, page, color);
		}break;

		case 103:	// 'g'
		{
			TG12864H3_WriteCharCol(0x18, x, page, color);
			TG12864H3_WriteCharCol(0xa4, x+1, page, color);
			TG12864H3_WriteCharCol(0xa4, x+2, page, color);
			TG12864H3_WriteCharCol(0xa4, x+3, page, color);
			TG12864H3_WriteCharCol(0x7c, x+4, page, color);
		}break;

		case 104:	// 'h'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x10, x+1, page, color);
			TG12864H3_WriteCharCol(0x08, x+2, page, color);
			TG12864H3_WriteCharCol(0x08, x+3, page, color);
			TG12864H3_WriteCharCol(0xf0, x+4, page, color);
		}break;

		case 105:	// 'i'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x88, x+1, page, color);
			TG12864H3_WriteCharCol(0xfa, x+2, page, color);
			TG12864H3_WriteCharCol(0x80, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 106:	// 'j'
		{
			TG12864H3_WriteCharCol(0x40, x, page, color);
			TG12864H3_WriteCharCol(0x80, x+1, page, color);
			TG12864H3_WriteCharCol(0x88, x+2, page, color);
			TG12864H3_WriteCharCol(0x7a, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 107:	// 'k'
		{
			TG12864H3_WriteCharCol(0xfe, x, page, color);
			TG12864H3_WriteCharCol(0x20, x+1, page, color);
			TG12864H3_WriteCharCol(0x50, x+2, page, color);
			TG12864H3_WriteCharCol(0x88, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 108:	// 'l'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x82, x+1, page, color);
			TG12864H3_WriteCharCol(0xfe, x+2, page, color);
			TG12864H3_WriteCharCol(0x80, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 109:	// 'm'
		{
			TG12864H3_WriteCharCol(0xf8, x, page, color);
			TG12864H3_WriteCharCol(0x08, x+1, page, color);
			TG12864H3_WriteCharCol(0x30, x+2, page, color);
			TG12864H3_WriteCharCol(0x08, x+3, page, color);
			TG12864H3_WriteCharCol(0xf0, x+4, page, color);
		}break;

		case 110:	// 'n'
		{
			TG12864H3_WriteCharCol(0xf8, x, page, color);
			TG12864H3_WriteCharCol(0x10, x+1, page, color);
			TG12864H3_WriteCharCol(0x08, x+2, page, color);
			TG12864H3_WriteCharCol(0x08, x+3, page, color);
			TG12864H3_WriteCharCol(0xf0, x+4, page, color);
		}break;

		case 111:	// 'o'
		{
			TG12864H3_WriteCharCol(0x70, x, page, color);
			TG12864H3_WriteCharCol(0x88, x+1, page, color);
			TG12864H3_WriteCharCol(0x88, x+2, page, color);
			TG12864H3_WriteCharCol(0x88, x+3, page, color);
			TG12864H3_WriteCharCol(0x70, x+4, page, color);
		}break;

		case 112:	// 'p'
		{
			TG12864H3_WriteCharCol(0xf8, x, page, color);
			TG12864H3_WriteCharCol(0x28, x+1, page, color);
			TG12864H3_WriteCharCol(0x28, x+2, page, color);
			TG12864H3_WriteCharCol(0x28, x+3, page, color);
			TG12864H3_WriteCharCol(0x10, x+4, page, color);
		}break;

		case 113:	// 'q'
		{
			TG12864H3_WriteCharCol(0x10, x, page, color);
			TG12864H3_WriteCharCol(0x28, x+1, page, color);
			TG12864H3_WriteCharCol(0x28, x+2, page, color);
			TG12864H3_WriteCharCol(0x30, x+3, page, color);
			TG12864H3_WriteCharCol(0xf8, x+4, page, color);
		}break;

		case 114:	// 'r'
		{
			TG12864H3_WriteCharCol(0xf8, x, page, color);
			TG12864H3_WriteCharCol(0x10, x+1, page, color);
			TG12864H3_WriteCharCol(0x08, x+2, page, color);
			TG12864H3_WriteCharCol(0x08, x+3, page, color);
			TG12864H3_WriteCharCol(0x10, x+4, page, color);
		}break;

		case 115:	// 's'
		{
			TG12864H3_WriteCharCol(0x90, x, page, color);
			TG12864H3_WriteCharCol(0xa8, x+1, page, color);
			TG12864H3_WriteCharCol(0xa8, x+2, page, color);
			TG12864H3_WriteCharCol(0xa8, x+3, page, color);
			TG12864H3_WriteCharCol(0x40, x+4, page, color);
		}break;

		case 116:	// 't'
		{
			TG12864H3_WriteCharCol(0x08, x, page, color);
			TG12864H3_WriteCharCol(0x7e, x+1, page, color);
			TG12864H3_WriteCharCol(0x88, x+2, page, color);
			TG12864H3_WriteCharCol(0x80, x+3, page, color);
			TG12864H3_WriteCharCol(0x40, x+4, page, color);
		}break;

		case 117:	// 'u'
		{
			TG12864H3_WriteCharCol(0x78, x, page, color);
			TG12864H3_WriteCharCol(0x80, x+1, page, color);
			TG12864H3_WriteCharCol(0x80, x+2, page, color);
			TG12864H3_WriteCharCol(0x40, x+3, page, color);
			TG12864H3_WriteCharCol(0xf8, x+4, page, color);
		}break;

		case 118:	// 'v'
		{
			TG12864H3_WriteCharCol(0x38, x, page, color);
			TG12864H3_WriteCharCol(0x40, x+1, page, color);
			TG12864H3_WriteCharCol(0x80, x+2, page, color);
			TG12864H3_WriteCharCol(0x40, x+3, page, color);
			TG12864H3_WriteCharCol(0x38, x+4, page, color);
		}break;

		case 119:	// 'w'
		{
			TG12864H3_WriteCharCol(0x78, x, page, color);
			TG12864H3_WriteCharCol(0x80, x+1, page, color);
			TG12864H3_WriteCharCol(0x60, x+2, page, color);
			TG12864H3_WriteCharCol(0x80, x+3, page, color);
			TG12864H3_WriteCharCol(0x78, x+4, page, color);
		}break;

		case 120:	// 'x'
		{
			TG12864H3_WriteCharCol(0x88, x, page, color);
			TG12864H3_WriteCharCol(0x50, x+1, page, color);
			TG12864H3_WriteCharCol(0x20, x+2, page, color);
			TG12864H3_WriteCharCol(0x50, x+3, page, color);
			TG12864H3_WriteCharCol(0x88, x+4, page, color);
		}break;

		case 121:	// 'y'
		{
			TG12864H3_WriteCharCol(0x18, x, page, color);
			TG12864H3_WriteCharCol(0xa0, x+1, page, color);
			TG12864H3_WriteCharCol(0xa0, x+2, page, color);
			TG12864H3_WriteCharCol(0xa0, x+3, page, color);
			TG12864H3_WriteCharCol(0x78, x+4, page, color);
		}break;

		case 122:	// 'z'
		{
			TG12864H3_WriteCharCol(0x88, x, page, color);
			TG12864H3_WriteCharCol(0xc8, x+1, page, color);
			TG12864H3_WriteCharCol(0xa8, x+2, page, color);
			TG12864H3_WriteCharCol(0x98, x+3, page, color);
			TG12864H3_WriteCharCol(0x88, x+4, page, color);
		}break;

		case 123:	// '{'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x10, x+1, page, color);
			TG12864H3_WriteCharCol(0x6c, x+2, page, color);
			TG12864H3_WriteCharCol(0x82, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 124:	// '|'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x00, x+1, page, color);
			TG12864H3_WriteCharCol(0xfe, x+2, page, color);
			TG12864H3_WriteCharCol(0x00, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 125:	// '}'
		{
			TG12864H3_WriteCharCol(0x00, x, page, color);
			TG12864H3_WriteCharCol(0x82, x+1, page, color);
			TG12864H3_WriteCharCol(0x6c, x+2, page, color);
			TG12864H3_WriteCharCol(0x10, x+3, page, color);
			TG12864H3_WriteCharCol(0x00, x+4, page, color);
		}break;

		case 126:	// '~'
		{
			TG12864H3_WriteCharCol(0x20, x, page, color);
			TG12864H3_WriteCharCol(0x10, x+1, page, color);
			TG12864H3_WriteCharCol(0x10, x+2, page, color);
			TG12864H3_WriteCharCol(0x20, x+3, page, color);
			TG12864H3_WriteCharCol(0x10, x+4, page, color);
		}break;
		/*

		case :	// ''
		{
			TG12864H3_WriteCharCol(0x, x, page, color);
			TG12864H3_WriteCharCol(0x, x+1, page, color);
			TG12864H3_WriteCharCol(0x, x+2, page, color);
			TG12864H3_WriteCharCol(0x, x+3, page, color);
			TG12864H3_WriteCharCol(0x, x+4, page, color);
		}break;
		*/

		default:	// not supported, print box
		{
			TG12864H3_WriteData(0xfe);
			TG12864H3_WriteData(0xfe);
			TG12864H3_WriteData(0xfe);
			TG12864H3_WriteData(0xfe);
			TG12864H3_WriteData(0xfe);
		}
	}
	TG12864H3_WriteCharCol(0x00, x+5, page, color);	//default last blank column of 6 x 8 character

	lcd_cs_1;
}
